import { EleventyHtmlBasePlugin } from "@11ty/eleventy";
import pluginEjs from '@11ty/eleventy-plugin-ejs';
import pluginRss from '@11ty/eleventy-plugin-rss';
import pluginSyntaxHighlight from "@11ty/eleventy-plugin-syntaxhighlight";
import { JSDOM } from 'jsdom';
import { DateTime } from "luxon";
import markdownItAnchor from 'markdown-it-anchor';

/** @param {import('@11ty/eleventy').UserConfig} eleventyConfig */
export default function(eleventyConfig) {
	eleventyConfig.setInputDirectory('content');

	if (process.env.NODE_ENV === 'localhost') {
		console.log('metadata override');
		eleventyConfig.addGlobalData('metadata.url', 'http://localhost:8080');
	}

	// Drafts
	eleventyConfig.addPreprocessor('drafts', '*', (data) => {
		if (data.draft && process.env.ELEVENTY_RUN_MODE === 'build') {
			return false;
		}
	});

	eleventyConfig.addPassthroughCopy("content/**/*.png");
	eleventyConfig.addPassthroughCopy("content/**/*.jpg");
	eleventyConfig.addPassthroughCopy("content/**/*.webp");
	eleventyConfig.addPassthroughCopy('content/**/*.css');
	eleventyConfig.addPassthroughCopy({ "static": "/" });

	// Run Eleventy when these files change:
	// https://www.11ty.dev/docs/watch-serve/#add-your-own-watch-targets

	// Watch content images for the image pipeline.
	eleventyConfig.addWatchTarget("content/**/*.{svg,webp,png,jpeg}");

	// Official plugins
	eleventyConfig.addBundle('css', {
		toFileDirectory: 'bundle',
	});
	eleventyConfig.addPlugin(pluginSyntaxHighlight, {
		preAttributes: { tabindex: 0 }
	});
	eleventyConfig.addPlugin(EleventyHtmlBasePlugin);
	eleventyConfig.addPlugin(pluginRss);
	eleventyConfig.addPlugin(pluginEjs, {
		outputFunctionName: 'print',
	});

	// Filters
	eleventyConfig.addFilter("readableDate", (dateObj, format, zone) => {
		// Formatting tokens for Luxon: https://moment.github.io/luxon/#/formatting?id=table-of-tokens
		return DateTime.fromJSDate(dateObj, { zone: zone || "utc" }).toFormat(format || "dd LLLL yyyy");
	});

	eleventyConfig.addFilter('htmlDateString', (dateObj) => {
		// dateObj input: https://html.spec.whatwg.org/multipage/common-microsyntaxes.html#valid-date-string
		return DateTime.fromJSDate(dateObj, {zone: 'utc'}).toFormat('yyyy-LL-dd');
	});

	// Get the first `n` elements of a collection.
	eleventyConfig.addFilter("head", (array, n) => {
		if(!Array.isArray(array) || array.length === 0) {
			return [];
		}
		if( n < 0 ) {
			return array.slice(n);
		}

		return array.slice(0, n);
	});

	// Return the smallest number argument
	eleventyConfig.addFilter("min", (...numbers) => {
		return Math.min.apply(null, numbers);
	});

	// Return all the tags used in a collection
	eleventyConfig.addFilter("getAllTags", collection => {
		let tagSet = new Set();
		for(let item of collection) {
			(item.data.tags || []).forEach(tag => tagSet.add(tag));
		}
		return Array.from(tagSet);
	});

	eleventyConfig.addFilter("filterTagList", function filterTagList(tags) {
		return (tags || []).filter(tag => ["all", "nav", "post", "posts"].indexOf(tag) === -1);
	});

	eleventyConfig.addFilter("getPostId", function getPostId(url) {
		const fragments = url.split('/').filter(f => f.trim());
		return fragments[fragments.length - 1];
	});

	eleventyConfig.addFilter("noTrailingSlash", function noTrailingSlash(url) {
		const length = url.length;
		if (url[length - 1] === '/') {
			return url.slice(0, length - 1);
		} else {
			return url;
		}
	});

	// https://stackoverflow.com/a/74005383
	eleventyConfig.addFilter('sections', (html) => {
		const dom = new JSDOM(html);
		const document = dom.window.document;
		const chunks = [];
		for (const h1 of document.querySelectorAll('h1')) {
			let chunk = '';
			let elem = h1;
			while (true) {
				chunk += elem.outerHTML;
				elem = document.evaluate(
					"following-sibling::*[1]", 
					elem,
					function() {},
					dom.window.XPathResult.FIRST_ORDERED_NODE_TYPE
				).singleNodeValue;
				if (!elem || elem.nodeName === 'H1') break;
			}
			chunks.push(chunk);
		}
		return chunks;
	});

	eleventyConfig.addShortcode("currentBuildDate", () => {
		return (new Date()).toISOString();
	})

	eleventyConfig.addNunjucksShortcode("webButton", function (src, alt, href) {
		const img = `<img src="${src}" alt="${alt}" title="${alt}" width=88 height=31 class="web-button">`;
		const a = href ? `<a href=${href}>${img}</a>` : img;
		return a;
	});

	// Customize Markdown settings
	eleventyConfig.amendLibrary('md', (mdLib) => {
		mdLib.use(markdownItAnchor, {
			permalink: markdownItAnchor.permalink.ariaHidden({
				placement: "after",
				class: "header-anchor",
				symbol: "#",
				ariaHidden: true,
			}),
			level: [1,2,3,4],
			slugify: eleventyConfig.getFilter("slugify")
		});
	});

	// Features to make your build faster (when you need them)

	// If your passthrough copy gets heavy and cumbersome, add this line
	// to emulate the file copy on the dev server. Learn more:
	// https://www.11ty.dev/docs/copy/#emulate-passthrough-copy-during-serve

	// eleventyConfig.setServerPassthroughCopyBehavior("passthrough");

	return {
		// Control which files Eleventy will process
		// e.g.: *.md, *.njk, *.html, *.liquid
		templateFormats: [
			"md",
			"njk",
			"html",
			"liquid",
		],

		// Pre-process *.md files with: (default: `liquid`)
		markdownTemplateEngine: "njk",

		// Pre-process *.html files with: (default: `liquid`)
		htmlTemplateEngine: "njk",

		// -----------------------------------------------------------------
		// Optional items:
		// -----------------------------------------------------------------

		// If your site deploys to a subdirectory, change `pathPrefix`.
		// Read more: https://www.11ty.dev/docs/config/#deploy-to-a-subdirectory-with-a-path-prefix

		// When paired with the HTML <base> plugin https://www.11ty.dev/docs/plugins/html-base/
		// it will transform any absolute URLs in your HTML to include this
		// folder name and does **not** affect where things go in the output folder.
		pathPrefix: "/",
	};
};
